/**
 * Copyright (C) 2008 Daniele Dellafiore <daniele.dellafiore@gmail.com>
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/MPL-1.1.html
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.phaedra.commons.file;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import jin.collection.core.Iter;
import jin.collection.core.Operation;
import net.phaedra.commons.tree.TreeExplorer;

/*
 * Created on 9-nov-2003

 */

/**
 * @author Daniele
 */
public class FileUtils {

   public static final void copy(final File file, final File newFile) {
      BufferedInputStream bis = null;
      BufferedOutputStream bos = null;
      try {
         bis = new BufferedInputStream(new FileInputStream(file));
         bos = new BufferedOutputStream(new FileOutputStream(newFile));
         copy(bis, bos);
      } catch (final IOException e) {
         throw new RuntimeIOException("Copy operation failed: ", e);
      } finally {
         try {
            bis.close();
            bos.close();
         } catch (final IOException e) {
            throw new RuntimeIOException("Error closing stream: ", e);
         } catch (final NullPointerException e) {
            throw new RuntimeIOException(
                  "Probably this streams were not initialized properly or the resource file does not exists: "
                        + file.getAbsolutePath() + " Error during copy operation: ", e);
         }
      }
   }

   public static final void copy(final File file, final Folder targetFolder) {
      copy(file, targetFolder, file.getName());
   }

   public static File copy(final File file, final Folder targetFolder, final String newFileName) {
      copy(file, new File(targetFolder, newFileName));
      return new File(targetFolder.getAbsolutePath(), newFileName);
   }

   public static void copy(final InputStream in, final File newFile) {
      try {
         final FileOutputStream out = new FileOutputStream(newFile);
         copy(in, out);
         in.close();
         out.close();
      } catch (final IOException e) {
         e.printStackTrace();
      }
   }

   private static final void copy(final InputStream in, final OutputStream out) throws IOException {

      synchronized (in) {
         synchronized (out) {
            final byte[] buffer = new byte[256];
            while (true) {
               final int bytesread = in.read(buffer);
               if (bytesread == -1) {
                  break;
               }
               out.write(buffer, 0, bytesread);
            }
         }
      }
   }

   /**
    * Not Recursive!
    */
   public static final List listSubFolders(final File folder) {
      return listFiles(folder, FileFilters.DirectoryFileFilter);
   }

   /**
    * Not Recursive!
    */
   public static final File[] subFolders(final File folder) {
      return folder.listFiles(FileFilters.DirectoryFileFilter);
   }

   /**
    * Not Recursive!
    */
   public static final List<File> listFiles(final File folder, final FileFilter fileFilter) {
      final File[] files = folder.listFiles(fileFilter);
      final List result = new LinkedList();
      if (files != null) {
         for (int i = 0; i < files.length; i++) {
            result.add(files[i]);
         }
      }
      return result;
   }

   public static List listSubFoldersRecursive(final File folderToScan) {
      return listSubFolders(folderToScan, true);
   }

   public static List listSubFolders(final File targetFolder, final boolean recursive) {
      return new TreeExplorer(FolderNode.class, targetFolder).scan(recursive);
   }

   public static final String getExtension(final File file) {
      return getExtension(file.getName());
   }

   public static String getExtension(final String filename) {
      String ext = new String(filename);
      final int i = filename.lastIndexOf('.');

      if (i > 0 && i < filename.length() - 1) {
         ext = filename.substring(i + 1);
      }
      return ext;
   }

   public static String fileNameWithNoExtension(final File file) {
      return fileNameWithNoExtension(file.getName());
   }

   public static String fileNameWithNoExtension(final String filename) {
      final String ext = getExtension(filename);
      final String result = filename.replaceAll("." + ext, "");
      return result;
   }

   /**
    * @param fileToCheck
    * @param folder
    * @return true if first File is in a subfolder of {@link #folder(File)}
    */
   public static boolean isInASubfolder(final File fileToCheck, final File folder) {
      File parentFile = fileToCheck.getParentFile();
      while (parentFile != null) {
         if (parentFile.equals(folder)) {
            return true;
         }
         parentFile = parentFile.getParentFile();
      }
      return false;
   }

   public static File getCurrentFolder() {
      return new File("");
   }

   public static String getCurrentFolderAbsolutePath() {
      return getCurrentFolder().getAbsolutePath().replace('\\', '/');
   }

   public static String normalizeFilePath(final String path) {
      String result = path.replace('\\', '/');
      result = result.replace("//", "/");
      return result;
   }

   public static List listFiles(final File folder) {
      return listFiles(folder, new FileFilter() {
         public boolean accept(final File pathname) {
            return true;
         }
      });
   }

   public static boolean isFolderEmpty(final File folder) {
      return folder.listFiles().length == 0;
   }

   // public static void setExtension(String ext) {
   // // TODO: implement
   // }

   public static File createFolder(final File parentFolder, final String folderName) {
      if (!parentFolder.exists()) {
         throw new RuntimeException(parentFolder + " does not exists");
      }
      if (!parentFolder.isDirectory()) {
         throw new RuntimeException(parentFolder + " is not a directory");
      }
      final File folder = new File(parentFolder.getAbsolutePath() + File.separator + folderName);
      folder.mkdir();
      return folder;
   }

   public static void deleteFoldersRecursively(final File dir) throws IOException {
      org.apache.commons.io.FileUtils.deleteDirectory(dir);
   }

   public static void deleteChildFoldersRecursively(final File dir) {
      if (dir.isDirectory()) {
         Iter.forEach(listSubFolders(dir), new Operation() {
            public void execute(final Object element) {
               final File folder = (File) element;
               try {
                  org.apache.commons.io.FileUtils.deleteDirectory(folder);
               } catch (final IOException e) {
                  throw new NonBlockingIOException("Error deleting directory: " + folder.getAbsolutePath(), e);
               }
            }
         });
      }
   }

   private static void delete(final File file) throws IOException {
      if (file.exists() && !file.delete()) {
         throw new IOException("Cannot delete " + file);
      }
   }

   /**
    * This is NOT recursive and does NOT count the folder passed as parameter
    */
   public static int countSubFolders(final File folder) {
      return subFolders(folder).length;
   }

   public static int countSubFoldersRecursive(final File folder) {
      return listSubFolders(folder, true).size();
   }

   public static boolean isLeaf(final File folder) {
      if (countSubFolders(folder) == 0) {
         return true;
      }
      return false;
   }

   public static File mkdirs(final File parentFolder, final String directoryPath) throws IOException {
      final File folder = new File(parentFolder, directoryPath);
      org.apache.commons.io.FileUtils.forceMkdir(folder);
      return folder;
   }

   public static void copy(final String originalFilePath, final String targetFilePath) {
      copy(new File(originalFilePath), new File(targetFilePath));
   }

   public static void deleteFoldersRecursively(final String dir) throws IOException {
      FileUtils.deleteFoldersRecursively(new File(dir));
   }

   public static URL toUrl(final String path) throws IOException {
      final URL[] ls = org.apache.commons.io.FileUtils.toURLs(new File[] { new File(path) });
      final URL url = ls[0];
      return url;
   }

}
