package net.phaedra.commons.values;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import net.phaedra.commons.utils.DateFormatUtil;
import net.phaedra.commons.utils.DateUtil;

import org.apache.commons.beanutils.ConversionException;
import org.apache.commons.beanutils.converters.DateConverter;
import org.apache.commons.lang.time.DateUtils;

public class Day implements Serializable, Comparable {

   private Date date;

   /**
    * Month here goes from 1 (January) to 12 (December) while Date range is
    * 0-11. Take care.
    * 
    * @param year
    * @param month
    * @param day
    */
   public Day(final int year, final int month, final int day) {
      final Calendar calendar = Calendar.getInstance();
      calendar.setLenient(false);
      calendar.set(year, month - 1, day);
      setTruncatedDate(calendar.getTime());
   }

   public static Day createFrom(final String year, final String month, final String day) {
      return new Day(Integer.parseInt(year), Integer.parseInt(month), Integer.parseInt(day));
   }

   public Day(final Date date) {
      setTruncatedDate(date);
   }

   public Day() {
      this(Calendar.getInstance().getTime());
   }

   public static Day fromString(String value, Locale locale) {
      if (value == null || value.isEmpty()) {
         return null;
      }
      DateConverter converter = new DateConverter();
      converter.setLocale(locale);
      try {
         Date date = (Date) converter.convert(Date.class, value);
         return new Day(date);
      } catch (Exception e) {
         throw new ConversionException(value);
      }
   }

   public static Day createFrom(final String source) {
      final String[] day = org.apache.commons.lang.StringUtils.split(source, "-");
      return createFrom(day[2], day[1], day[0]);
   }

   private void setTruncatedDate(final Date date) {
      final Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      final Calendar truncatedalendar = DateUtils.truncate(calendar, Calendar.DAY_OF_MONTH);
      this.date = truncatedalendar.getTime();
   }

   public boolean before(final Day secondDay) {
      return date.before(secondDay.date);
   }

   public boolean after(final Day secondDay) {
      return date.after(secondDay.date);
   }

   public Date asDate() {
      return date;
   }

   @Override
   public int hashCode() {
      return date.hashCode();
   }

   @Override
   public String toString() {
      final Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      final int humanReadableMonth = calendar.get(Calendar.MONTH) + 1;
      final String dayString = calendar.get(Calendar.DAY_OF_MONTH) + "-" + humanReadableMonth + "-"
            + calendar.get(Calendar.YEAR);
      return dayString;
   }

   @Override
   public boolean equals(final Object obj) {
      if (this == obj) {
         return true;
      }
      if (obj == null) {
         return false;
      }
      if (getClass() != obj.getClass()) {
         return false;
      }
      final Day other = (Day) obj;
      if (date == null) {
         if (other.date != null) {
            return false;
         }
      } else if (!date.equals(other.date)) {
         return false;
      }
      return true;
   }

   public Day addDays(final int days) {
      final Date newDate = DateUtils.addDays(date, days);
      final Calendar calendar = Calendar.getInstance();
      calendar.setTime(newDate);
      final int humanReadableMonth = calendar.get(Calendar.MONTH) + 1;
      return new Day(calendar.get(Calendar.YEAR), humanReadableMonth, calendar.get(Calendar.DAY_OF_MONTH));
   }

   public int toDate(final Day expireDate) {
      return DateUtil.daysBetween(date, expireDate.date);
   }

   public Day subtractDays(final int days) {
      final Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      calendar.add(Calendar.DAY_OF_MONTH, -days);
      final int humanReadableMonth = calendar.get(Calendar.MONTH) + 1;
      return new Day(calendar.get(Calendar.YEAR), humanReadableMonth, calendar.get(Calendar.DAY_OF_MONTH));
   }

   public static Day today() {
      return new Day(new Date());
   }

   public Time asTime() {
      final Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      final int day = calendar.get(Calendar.DAY_OF_MONTH);
      final int humanReadableMonth = calendar.get(Calendar.MONTH) + 1;
      final int year = calendar.get(Calendar.YEAR);
      return new Time(year, humanReadableMonth, day, 12, 0, 0);
   }

   /**
    * Format using default machine locale
    */
   public String format() {
      return format(Locale.getDefault());
   }

   public String format(final Locale locale) {
      final SimpleDateFormat sdf = new SimpleDateFormat(DateFormatUtil.DAY_FORMAT, locale);
      return sdf.format(asDate());
   }

   @Override
   public int compareTo(final Object o) {
      final Day other = (Day) o;
      return date.compareTo(other.asDate());
   }

   public Integer getDay() {
      final Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      final int day = calendar.get(Calendar.DAY_OF_MONTH);
      return day;
   }

   public Day increment(final int amount) {
      return new Day(DateUtils.addDays(date, amount));
   }

}
